;;; Directory Local Variables
;;; See Info node `(emacs) Directory Variables' for more information.

((nil .
  ((fill-column . 79)
   (indent-tabs-mode)
   (show-trailing-whitespace . t)
   (whitespace-style face trailing lines-tail space-before-tab indentation)
   (whitespace-newline . t)))
 (c++-mode .
  ((tab-width . 4)
   (whitespace-action warn-read-only auto-cleanup)))
 (cmake-mode .
  ((whitespace-action warn-read-only auto-cleanup))))
